import { world, BlockPermutation, ItemStack, GameMode, system } from "@minecraft/server";

// Helper function to decrement an item stack
function decrementStack(item, player) {
	if (player.getGameMode() === "survival") {
		if (item.amount > 1) {
			item.amount -= 1;
			player.getComponent("equippable").setEquipment("Mainhand", item);
		} else {
			player.getComponent("equippable").setEquipment("Mainhand", new ItemStack("minecraft:air"));
		}
	}
}

function pot_center(block) {
	return {
		x: block.location.x + 0.5,
		y: block.location.y + 0.3,
		z: block.location.z + 0.5
	};
}

const seeds_can_place_on = {
	dirt: ["minecraft:oak_sapling", "minecraft:spruce_sapling", "minecraft:birch_sapling",
		"minecraft:jungle_sapling", "minecraft:acacia_sapling", "minecraft:dark_oak_sapling",
		"minecraft:cherry_sapling", "minecraft:pale_oak_sapling", "minecraft:wheat_seeds",
		"minecraft:carrot", "minecraft:potato", "minecraft:beetroot_seeds",
		"minecraft:melon_seeds", "minecraft:pumpkin_seeds", "minecraft:poppy",
		"minecraft:blue_orchid", "minecraft:azure_bluet", "minecraft:red_tulip",
		"minecraft:orange_tulip", "minecraft:white_tulip", "minecraft:pink_tulip",
		"minecraft:oxeye_daisy", "minecraft:cornflower", "minecraft:lily_of_the_valley",
		"minecraft:dandelion", "minecraft:bamboo", "minecraft:sugar_cane",
		"minecraft:rose_bush", "minecraft:lilac", "minecraft:peony",
		"minecraft:brown_mushroom", "minecraft:red_mushroom"],

	grass_block: ["minecraft:oak_sapling", "minecraft:spruce_sapling", "minecraft:birch_sapling",
		"minecraft:jungle_sapling", "minecraft:acacia_sapling", "minecraft:dark_oak_sapling",
		"minecraft:cherry_sapling", "minecraft:pale_oak_sapling", "minecraft:wheat_seeds",
		"minecraft:carrot", "minecraft:potato", "minecraft:beetroot_seeds",
		"minecraft:melon_seeds", "minecraft:pumpkin_seeds", "minecraft:poppy",
		"minecraft:blue_orchid", "minecraft:azure_bluet", "minecraft:red_tulip",
		"minecraft:orange_tulip", "minecraft:white_tulip", "minecraft:pink_tulip",
		"minecraft:oxeye_daisy", "minecraft:cornflower", "minecraft:lily_of_the_valley",
		"minecraft:dandelion", "minecraft:bamboo", "minecraft:sugar_cane",
		"minecraft:rose_bush", "minecraft:lilac", "minecraft:peony",
		"minecraft:brown_mushroom", "minecraft:red_mushroom"],
	farmland: ["minecraft:oak_sapling", "minecraft:spruce_sapling", "minecraft:birch_sapling",
		"minecraft:jungle_sapling", "minecraft:acacia_sapling", "minecraft:dark_oak_sapling",
		"minecraft:cherry_sapling", "minecraft:pale_oak_sapling", "minecraft:wheat_seeds",
		"minecraft:carrot", "minecraft:potato", "minecraft:beetroot_seeds",
		"minecraft:melon_seeds", "minecraft:pumpkin_seeds", "minecraft:poppy",
		"minecraft:blue_orchid", "minecraft:azure_bluet", "minecraft:red_tulip",
		"minecraft:orange_tulip", "minecraft:white_tulip", "minecraft:pink_tulip",
		"minecraft:oxeye_daisy", "minecraft:cornflower", "minecraft:lily_of_the_valley",
		"minecraft:dandelion", "minecraft:bamboo", "minecraft:sugar_cane",
		"minecraft:rose_bush", "minecraft:lilac", "minecraft:peony",
		"minecraft:brown_mushroom", "minecraft:red_mushroom"],
	sand: ["minecraft:cactus", "minecraft:sugar_cane"],
	mycelium: ["minecraft:oak_sapling", "minecraft:spruce_sapling", "minecraft:birch_sapling",
		"minecraft:jungle_sapling", "minecraft:acacia_sapling", "minecraft:dark_oak_sapling",
		"minecraft:cherry_sapling", "minecraft:pale_oak_sapling", "minecraft:wheat_seeds",
		"minecraft:carrot", "minecraft:potato", "minecraft:beetroot_seeds",
		"minecraft:melon_seeds", "minecraft:pumpkin_seeds", "minecraft:poppy",
		"minecraft:blue_orchid", "minecraft:azure_bluet", "minecraft:red_tulip",
		"minecraft:orange_tulip", "minecraft:white_tulip", "minecraft:pink_tulip",
		"minecraft:oxeye_daisy", "minecraft:cornflower", "minecraft:lily_of_the_valley",
		"minecraft:dandelion", "minecraft:bamboo", "minecraft:sugar_cane",
		"minecraft:rose_bush", "minecraft:lilac", "minecraft:peony",
		"minecraft:brown_mushroom", "minecraft:red_mushroom"],
	crimson_nylium: ["minecraft:crimson_fungus"],
	warped_nylium: ["minecraft:warped_fungus"],
	soul_sand: ["minecraft:nether_wart"],
	end_stone: ["minecraft:chorus_flower"]
};

const soil_drop = {
	dirt: "minecraft:dirt",
	grass_block: "minecraft:grass_block",
	sand: "minecraft:sand",
	mycelium: "minecraft:mycelium",
	soul_sand: "minecraft:soul_sand",
	farmland: "minecraft:farmland",
	end_stone: "minecraft:end_stone"
};

const wheat_type = ["minecraft:wheat_seeds", "minecraft:carrot", "minecraft:potato",
	"minecraft:beetroot_seeds", "minecraft:nether_wart"];

const block_type = ["minecraft:pumpkin_seeds", "minecraft:melon_seeds"]

const flower_type = ["minecraft:poppy", "minecraft:blue_orchid", "minecraft:azure_bluet",
	"minecraft:red_tulip", "minecraft:orange_tulip", "minecraft:white_tulip",
	"minecraft:pink_tulip", "minecraft:oxeye_daisy", "minecraft:cornflower",
	"minecraft:lily_of_the_valley", "minecraft:dandelion"];

const bamboo_type = ["minecraft:bamboo"];
const double_flower_type = ["minecraft:sugar_cane", "minecraft:rose_bush", "minecraft:lilac",
	"minecraft:peony"];
const cactus_type = ["minecraft:cactus"];
const mushroom_type = ["minecraft:brown_mushroom", "minecraft:red_mushroom", "minecraft:crimson_fungus",
	"minecraft:warped_fungus"]
const chorus_flower_type = ["minecraft:chorus_flower"];


world.beforeEvents.worldInitialize.subscribe((initEvent) => {
	initEvent.blockComponentRegistry.registerCustomComponent("botany:on_interact", {
		onPlayerInteract: (event) => {
			const { player, block } = event;

			const equippable = player.getComponent("equippable");
			const mainHandItem = equippable.getEquipment("Mainhand");


			const soil_type = block.permutation.getState("pot:soil_type");
			const seeds_equals_pot = seeds_can_place_on[soil_type] || [];


			if (player.isSneaking && block.permutation.getState("pot:with_crop") === false) {

				const soil_type = block.permutation.getState("pot:soil_type");

				if (soil_drop[soil_type]) {
					const soil_item = soil_drop[soil_type];

					const centerLocation = pot_center(block);

					block.dimension.spawnItem(new ItemStack(soil_item), centerLocation);

					const newPermutation = block.permutation
						.withState("pot:soil_type", "none")
						.withState("pot:with_crop", false);
					block.setPermutation(newPermutation);
				}
			}
			if (seeds_equals_pot.includes(mainHandItem.typeId) && block.permutation.getState("pot:with_crop") === false) {
				player.playSound("dig.grass");
				decrementStack(mainHandItem, player);

				const newPermutation = block.permutation.withState("pot:with_crop", true);
				block.setPermutation(newPermutation);

				if (mainHandItem.typeId.endsWith("_sapling")) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const saplingEntity = block.dimension.spawnEntity("botany_reborn:sapling_type", centerLocation);

					const saplingEvent = mainHandItem.typeId.replace("minecraft:", "");
					saplingEntity.runCommand(`event entity @s ${saplingEvent}`);
				}
				if (wheat_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const wheatEntity = block.dimension.spawnEntity("botany_reborn:wheat_type", centerLocation);

					const wheatEvent = mainHandItem.typeId.replace("minecraft:", "");
					wheatEntity.runCommand(`event entity @s ${wheatEvent}`);
				}
				if (block_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const blocktopEntity = block.dimension.spawnEntity("botany_reborn:block_top_type", centerLocation);

					const blocktopEvent = mainHandItem.typeId.replace("minecraft:", "");
					blocktopEntity.runCommand(`event entity @s ${blocktopEvent}`);

					const blocksideEntity = block.dimension.spawnEntity("botany_reborn:block_side_type", centerLocation);

					const blocksideEvent = mainHandItem.typeId.replace("minecraft:", "");
					blocksideEntity.runCommand(`event entity @s ${blocksideEvent}`);
				}
				if (flower_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const flowerEntity = block.dimension.spawnEntity("botany_reborn:flower_type", centerLocation);

					const flowerEvent = mainHandItem.typeId.replace("minecraft:", "");
					flowerEntity.runCommand(`event entity @s ${flowerEvent}`);
				}
				if (bamboo_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const bambooEntity = block.dimension.spawnEntity("botany_reborn:bamboo_type", centerLocation);

					const bambooEvent = mainHandItem.typeId.replace("minecraft:", "");
					bambooEntity.runCommand(`event entity @s ${bambooEvent}`);
				}
				if (double_flower_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const dbflowerEntity = block.dimension.spawnEntity("botany_reborn:double_flower_type", centerLocation);

					const dbflowerEvent = mainHandItem.typeId.replace("minecraft:", "");
					dbflowerEntity.runCommand(`event entity @s ${dbflowerEvent}`);

					const flowerEntity = block.dimension.spawnEntity("botany_reborn:flower_type", centerLocation);

					const flowerEvent = mainHandItem.typeId.replace("minecraft:", "");
					flowerEntity.runCommand(`event entity @s ${flowerEvent}`);
				}
				if (cactus_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const cactussideEntity = block.dimension.spawnEntity("botany_reborn:cactus_side_type", centerLocation);

					const cactussideEvent = mainHandItem.typeId.replace("minecraft:", "");
					cactussideEntity.runCommand(`event entity @s ${cactussideEvent}`);

					const cactustopEntity = block.dimension.spawnEntity("botany_reborn:cactus_top_type", centerLocation);

					const cactustopEvent = mainHandItem.typeId.replace("minecraft:", "");
					cactustopEntity.runCommand(`event entity @s ${cactustopEvent}`);
				}
				if (mushroom_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const mushroomEntity = block.dimension.spawnEntity("botany_reborn:mushroom_type", centerLocation);

					const mushroomEvent = mainHandItem.typeId.replace("minecraft:", "");
					mushroomEntity.runCommand(`event entity @s ${mushroomEvent}`);
				}
				if (chorus_flower_type.includes(mainHandItem.typeId)) {
					player.playSound("dig.grass");

					const centerLocation = pot_center(block);

					const chorusflowerEntity = block.dimension.spawnEntity("botany_reborn:chorus_flower_type", centerLocation);

					const chorusflowerEvent = mainHandItem.typeId.replace("minecraft:", "");
					chorusflowerEntity.runCommand(`event entity @s ${chorusflowerEvent}`);
				}


			}

			if (mainHandItem.typeId === "minecraft:dirt" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.gravel");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'dirt')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);

			}
			if (mainHandItem.typeId === "minecraft:grass_block" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.grass");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'grass_block')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:farmland" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.gravel");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'farmland')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:mycelium" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.grass");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'mycelium')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:sand" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.sand");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'sand')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:clay" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.gravel");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'clay')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:crimson_nylium" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.nylium");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'crimson_nylium')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:warped_nylium" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.nylium");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'warped_nylium')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:soul_sand" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.soul_sand");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'soul_sand')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:end_stone" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("dig.stone");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'end_stone')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				decrementStack(mainHandItem, player);
			}
			if (mainHandItem.typeId === "minecraft:water_bucket" && block.permutation.getState('pot:soil_type') === "none" && block.permutation.getState('pot:with_crop') === false) {
				player.playSound("bucket.empty_water");

				const newPermutation = block.permutation
					.withState('pot:soil_type', 'water')
					.withState('pot:with_crop', false);
				block.setPermutation(newPermutation);

				player.runCommand(`replaceitem entity @s[m=!c] slot.weapon.mainhand 0 bucket`);

			}

			// Check if player is sneaking and with_crop is false

		},
	});
});

world.afterEvents.playerBreakBlock.subscribe((eventData) => {
	const { brokenBlockPermutation, block, dimension, player } = eventData;

	// Check if the broken block is a botany_reborn:pot
	if (brokenBlockPermutation.type.id === "botany_reborn:pot") {
		// Check if the player is in Survival mode
		if (player.getGameMode() === "survival") {
			const soil_type = brokenBlockPermutation.getState("pot:soil_type");

			// Use the helper function to get the center location of the block
			const centerLocation = pot_center(block);

			// Spawn the pot item at the center location
			dimension.spawnItem(new ItemStack("botany_reborn:pot"), centerLocation);

			// Spawn the soil item if the soil type is mapped
			if (soil_drop[soil_type]) {
				dimension.spawnItem(new ItemStack(soil_drop[soil_type]), centerLocation);
			}

			// Kill any entity directly above the block
			player.runCommand(`function items_drop`);
		}

		// Check if the player is in Creative mode
		if (player.getGameMode() === "creative") {
			// Kill any entity directly above the block with a slightly larger radius
			player.runCommand(`execute at @e[x=${block.location.x},y=${block.location.y},z=${block.location.z},r=2] if block ~~~ air run event entity @e[r=0.001] despawn`);
		}
	}
});
const SCALE_TAG = "botany_can_grow"; // Tag for scalable entities
const SCOREBOARD_NAME = "botany_percent"; // Name of the scoreboard for scaling

// Function to scale entities based on their scoreboard
system.run(() => {
	system.runInterval(() => {
		const dimension = world.getDimension("overworld"); // Overworld dimension

		// Define the entity types to scale
		const entityTypes = [
			"botany_reborn:sapling_type",
			"botany_reborn:wheat_type",
			"botany_reborn:block_top_type",
			"botany_reborn:block_side_type",
			"botany_reborn:flower_type",
			"botany_reborn:bamboo_type",
			"botany_reborn:double_flower_type",
			"botany_reborn:cactus_side_type",
			"botany_reborn:cactus_top_type",
			"botany_reborn:mushroom_type",
			"botany_reborn:chorus_flower_type"
		];

		for (const type of entityTypes) {
			const entities = dimension.getEntities({ type });

			for (const entity of entities) {
				try {
					// Retrieve the scoreboard value for the entity
					const scoreboard = world.scoreboard.getObjective(SCOREBOARD_NAME);
					if (!scoreboard) {
						console.warn(`Scoreboard "${SCOREBOARD_NAME}" not found.`);
						continue;
					}

					const score = scoreboard.getScore(entity);
					if (score === undefined || score === null) {
						console.warn(`Entity ${entity.id} has no score for "${SCOREBOARD_NAME}".`);
						continue;
					}

					// Calculate scale based on the score
					const scaleValue = score / 100; // Clamp between 0.5 and 3.0

					// Check for scale component and update it
					const scaleComponent = entity.getComponent("minecraft:scale");
					if (scaleComponent) {
						scaleComponent.value = scaleValue; // Modify the scale value
					} else {
						console.warn(`Entity ${entity.id} does not have a scale component.`);
					}
				} catch (error) {
					console.error(`Error scaling entity: ${error.message}`);
				}
			}
		}
	}, 1); // Run every 20 ticks (1 second)
});